
// needs g++-13.1

#include <iostream>
#include <chrono>
#include <format>
//#include <string>

int main() {
    using namespace std::chrono;
    // test
    const zoned_time cur_time{ current_zone(), system_clock::now() };
    std::cout << cur_time << '\n';

    // local time_point
    auto now = current_zone()->to_local(system_clock::now());
    std::cout << "TZ: " << current_zone()->name() << "\n";

    // convert time_point to days
    auto ld = floor<days>(now);

    // convert days to {y, m, d} calendar
    year_month_day date{ld};
    std::cout << "date: " << date << "\n";

    // time since midnight
    hh_mm_ss hms{now - ld};
    std::cout << "time: " << hms << "\n";

    // print date
    std::cout << date << "\n";
    std::cout << std::format("{:%d.%m.%Y}", date) << "\n";
    std::cout << std::format("{:%e %B %Y}", date) << "\n";

    // difference of dates in days (NO months, hours etc.)
    auto refdate{2023y/January/1};
    std::cout << (sys_days{date} - sys_days{refdate})
              << " since " << refdate << "\n";

    auto refdate2{1900y/January/1};
    std::cout << (sys_days{date} - sys_days{refdate2})
              << " since " << refdate2 << "\n";

    // seconds -> hh:mm:ss
    auto ss{12345s};
    std::cout << ss << " == " << hh_mm_ss{ss} << '\n';
}
/*
possible output:
===========================================================
2023-04-27 15:20:39.393886493 UTC
TZ: Etc/UTC
date: 2023-04-27
time: 15:20:39.394037501
2023-04-27
27.04.2023
27 April 2023
116d since 2023-01-01
45041d since 1900-01-01
12345s == 03:25:45
===========================================================
*/
