
// C++11
// std::chrono usage: test overhead of high_resolution_clock

#include <chrono>
#include <array>
#include <iostream>

int main() {
    using namespace std::chrono;
    std::cout << "\nhigh_resolution_clock:\n";

#if 1
    // busy wait (for CPU frequency boost only)
    for(long l=0; l<10000000; ++l)
       std::cout << ""; // do nothing
#endif

    // find possible overhead (unit: ns)
    const int MAX{100};
//    std::array<int,MAX+1> h{0};
    int h[MAX+1] = {0,};
    const int N=10000;
    for(int i=0; i<N; ++i) {
        auto hrc1 = high_resolution_clock::now();
        auto hrc2 = high_resolution_clock::now();
        auto delta = hrc2-hrc1;

        int x = nanoseconds(delta).count(); // conversion
        if(x < 0 || x > MAX) x=MAX;
        ++h[x];
    }
    // print
    std::cout << "Histogram for " << N << " samples:\n";
    for(int i=0; i<MAX; ++i) {
        int count = h[i];
        if(count==0) continue; // ignore zero count bins
        std::cout << "[" << i << "] = " << count << "\n";
    }
    std::cout << "[not in <0," << MAX << ">] = " << h[MAX] << "\n";
}
