
// simple example of C++ threads

#include <thread>
#include <mutex>
#include <chrono>
#include <iostream>

// todo: thread_pool

std::mutex display_mutex;
using namespace std::chrono_literals;   // 1s

void f()
{
    auto start = std::chrono::high_resolution_clock::now();
    display_mutex.lock();
    std::cout << "Thread "<<std::this_thread::get_id()<<" print1\n";
    display_mutex.unlock();
    for(unsigned u=1; u!=0; ++u)
        ;
    std::this_thread::sleep_for(1s);
    display_mutex.lock();
    std::cout << "Thread "<<std::this_thread::get_id()<<" print2\n";
    display_mutex.unlock();
    auto end = std::chrono::high_resolution_clock::now();
    // compute and print duration in ms
    std::chrono::duration<double, std::milli> elapsed = end-start;
    display_mutex.lock();
    std::cout << "Duration: " << elapsed.count() << " ms\n";
    display_mutex.unlock();
}

int main() try {
    std::cout << "main print1: NCPU="<<std::thread::hardware_concurrency()<<"\n";
    std::thread t(f);   // f() executes in separate thread
    display_mutex.lock();
    std::cout << "main print2\n";
    display_mutex.unlock();
    for(unsigned u=1; u<=1'000'000'000; ++u)
        ;
    display_mutex.lock();
    std::cout << "main print3: joinable="<<t.joinable()<<"\n";
    display_mutex.unlock();
    std::this_thread::sleep_for(1s);
    t.join();           // wait for thread completion
    std::cout << "main print4: joinable="<<t.joinable()<<"\n";
}catch(...) {
    //std::ofstream("a-log.txt", std::ios::app) << "uncaught exception\n";
    std::cerr << "uncaught exception\n";
}
